"""
Purpose of this script is to allow the automated delineation of watersheds from a Digital Elevation Model.
The script tool makes use of several tools from the Spatial Analyst Hydrology toolset.

This script tool was developed by UNI Geography/Computer Science student Dwain Caldwell as part of his
GIS II course project in Spring 2013.

This tool requires a Digital Elevation Model and a set of pour points (i.e. outlet of a watershed/catchment area)
as a raster dataset. 
"""
"""

file:GISII.py
author: dwain caldwell
description: script to automate the tools required to create a
             basin and watershed using the ArcGIS Spatial Analysis
             hydrology toolset.
             
"""
#import necessary modules
from arcpy import sa
from arcpy import env

#set workspace
env.workspace = arcpy.GetParameterAsText(0)

#read in original elevation file
originalDEM = arcpy.GetParameterAsText(1)
arcpy.AddMessage("Elevation File: " + originalDEM+ "\n")

"""Fill tool: this will be used to fill in areas of the DEM that are not true depressions.
              A z-value may be specified by the user which will enhance user control
              over what may be considered a true depression."""
#read in fill options
useFill = arcpy.GetParameterAsText(4)
zValue = arcpy.GetParameterAsText(5)
#if zValue given by user, change to integer
if not zValue == "":
    zValue = int(zValue)
if useFill != "false":
    arcpy.AddMessage("Filling DEM\n")
    #Conditional check to see if user gave a z-value. If given, use the value.
    if zValue != "":
        outFill = sa.Fill(originalDEM,zValue)
    else:
        outFill = sa.Fill(originalDEM)
else:
    arcpy.AddMessage("You have chosen not to fill the DEM\n")
#this line is necessary to save the raster 
outFill.save('DEM_Fill')


#run flow direction tool and save raster
forceFlow = arcpy.GetParameterAsText(6)
arcpy.AddMessage("Running Flow Direction Tool\n")
if forceFlow != "" and useFill == "true":             
    outFlowD = sa.FlowDirection(outFill,forceFlow)
elif forceFlow != "":
    outFlowD = sa.FlowDirection(originalDEM,forceFlow)
elif useFill == "true":
    outFlowD = sa.FlowDirection(outFill)
else:
    outFlowD = sa.FlowDirection(originalDEM)
    
outFlowD.save('DEM_FlowD')

#run flow accumulation tool and save raster
dataType = arcpy.GetParameterAsText(7)
arcpy.AddMessage("Running Flow Accumulation Tool\n")
if dataType != "":
    outFlowA = sa.FlowAccumulation(outFlowD,"",dataType)
else:
    outFlowA = sa.FlowAccumulation(outFlowD)

outFlowA.save('DEM_FlowA')

#Build a stream network and save raster
threshold = arcpy.GetParameterAsText(8)
arcpy.AddMessage("Running Con Tool to build stream network\n")
if threshold == "":
    threshold = "2000"
streamNetwork =sa.Con(outFlowA,1,"","Value > "+threshold,)
streamNetwork.save("streamNetwork")

#use stream link tool and save raster
arcpy.AddMessage("Running Stream Link Tool\n")
streamLink = sa.StreamLink(streamNetwork,outFlowD)
streamLink.save("streamLink")

#use stream order tool and save raster
orderMethod = arcpy.GetParameterAsText(9)
arcpy.AddMessage("Running Stream Order Tool\n")
if orderMethod == "":
    orderMethod = "STRAHLER"
streamOrder = sa.StreamOrder(streamNetwork,outFlowD,orderMethod)
streamOrder.save("streamOrder")

#Convert Stream to Feature
arcpy.AddMessage("Converting Streams to Features\n")
streams = arcpy.GetParameterAsText(10)
simplify = arcpy.GetParameterAsText(11)
if simplify =="true":
    simplify = "SIMPLIFY"
else:
    simplify = "NO_SIMPLIFY"
sa.StreamToFeature(streamNetwork,outFlowD,streams,simplify)

#Run Basin tool and save raster
arcpy.AddMessage("Creating a basin raster\n")
basin = sa.Basin(outFlowD)
basin.save("basin")

#Run Snap Pour Point tooland save raster
arcpy.AddMessage("Running Snap Pour Point tool\n")
pourPoints = arcpy.GetParameterAsText(2)
tolerance = arcpy.GetParameterAsText(3)
if not tolerance == "":
    tolerance = int(tolerance)

#set env settings so union of inputs are used
env.extent = "MAXOF"

snapPts = sa.SnapPourPoint(pourPoints,outFlowA,tolerance,"VALUE")
snapPts.save("SnapPourPts")

#run watershed tool and save raster
arcpy.AddMessage("Running Watershed tool\n")
watershed = sa.Watershed(outFlowD,snapPts)
watershed.save("Watershed")
